@@ -221,7 +221,6 @@ int main(int argc, char *argv[])
 	__connman_stats_init();
 
 	__connman_resolver_init(option_dnsproxy);
-	__connman_dnsproxy_init(option_dnsproxy);
 	__connman_ipconfig_init();
 	__connman_rtnl_init();
 	__connman_task_init();
@@ -257,7 +256,6 @@ int main(int argc, char *argv[])
 	__connman_task_cleanup();
 	__connman_rtnl_cleanup();
 	__connman_ipconfig_cleanup();
-	__connman_dnsproxy_cleanup();
 	__connman_resolver_cleanup();
 
 	__connman_stats_cleanup();
